import unittest
import subprocess
import os

class MainTest(unittest.TestCase):
    def test_cpp(self):
        print("\n\nTesting C++ code...")
        subprocess.check_call(os.path.join(os.path.dirname(
            os.path.relpath(__file__)), 'bin', 'pinetree_test'))
        print()  # for prettier output


if __name__ == '__main__':
    unittest.main()



import unittest
import pinetree as pt


class TestMobileElementManagerMethods(unittest.TestCase):

    def test_init(self):
        manager = pt.MobileElementManager([])

    def test_insert(self):
        pass

    def test_delete(self):
        pass

    def test_choose(self):
        pass

    def test_valid_index(self):
        pass

    def test_get_pol(self):
        pass

    def test_get_attached(self):
        pass

    def test_update_propensity(self):
        pass



import pandas
import math

my_df1 = pd.read_csv("run0_counts.tsv", sep="\t", names=['time', 'protein', 'count'])
my_df2 = pd.read_csv("run1_counts.tsv", sep="\t", names=['time', 'protein', 'count'])

all_df = pd.concat(['my_df1', 'my_df2'])

all_df['time'] = all_df['time'].apply(math.floor)

all_df = all_df.groupby(['time', 'protein'])

all_df.agg({'count':['mean', 'std']}).reset_index()




import pinetree as pt


def execute(output):

    sim = pt.Model(cell_volume=8e-16)
    sim.seed(34)
    sim.add_polymerase(name="rnapol", copy_number=1, speed=40, footprint=10)
    sim.add_ribosome(copy_number=1, speed=30, footprint=10)

    plasmid = pt.Genome(name="T7", length=605)

    plasmid.add_promoter(name="phi1", start=1, stop=10,
                         interactions={"rnapol": 2e8})
    plasmid.add_terminator(name="t1", start=604, stop=605,
                           efficiency={"rnapol": 1.0})

    plasmid.add_gene(name="rnapol", start=26, stop=225,
                     rbs_start=(26 - 15), rbs_stop=26, rbs_strength=1e7)
    plasmid.add_gene(name="proteinX", start=241, stop=280,
                     rbs_start=(241 - 15), rbs_stop=241, rbs_strength=1e7)
    plasmid.add_gene(name="proteinY", start=296, stop=595,
                     rbs_start=(296 - 15), rbs_stop=296, rbs_strength=1e7)

    sim.register_genome(plasmid)

    sim.simulate(time_limit=40, time_step=1, output=output + "_counts.tsv")


if __name__ == "__main__":
    execute("single_gene")



import pinetree as pt


def execute(output):

    sim = pt.Model(cell_volume=8e-16)
    sim.seed(34)
    sim.add_polymerase(name="rnapol", copy_number=10, speed=30, footprint=10)
    sim.add_ribosome(copy_number=100,
                     speed=20, footprint=10)

    plasmid = pt.Genome(name="T7", length=305,
                        transcript_degradation_rate=1e-2,
                        transcript_degradation_rate_ext=1e-2,
                        rnase_footprint=9,
                        rnase_speed=20)

    # plasmid = pt.Genome(name="T7", length=305)

    plasmid.add_promoter(name="phi1", start=1, stop=10,
                         interactions={"rnapol": 2e8})
    plasmid.add_terminator(name="t1", start=304, stop=305,
                           efficiency={"rnapol": 1.0})
    plasmid.add_gene(name="proteinX", start=30, stop=99,
                     rbs_start=(30 - 10), rbs_stop=30, rbs_strength=1e7)
    # plasmid.add_promoter(name="phi2", start=100, stop=109,
    #                      interactions={"rnapol": 2e8})
    # plasmid.add_terminator(name="t1", start=98, stop=99,
    #                        efficiency={"rnapol": 1.0})
    # plasmid.add_rnase_site(100, 110)
    plasmid.add_gene(name="proteinY", start=120, stop=199,
                     rbs_start=(120 - 10), rbs_stop=120, rbs_strength=1e7)
    # plasmid.add_promoter(name="phi3", start=200, stop=209,
    #                     interactions={"rnapol": 2e8})
    plasmid.add_gene(name="proteinZ", start=220, stop=300,
                     rbs_start=(220 - 10), rbs_stop=220, rbs_strength=1e7)

    sim.register_genome(plasmid)

    sim.simulate(time_limit=500, time_step=1, output=output + "counts.tsv")


if __name__ == "__main__":
    execute("three_genes_rnase_")



#! /usr/bin/env python3

import os
import re
import sys
import sysconfig
import platform
import subprocess

from distutils.version import LooseVersion
from setuptools import setup, Extension, find_packages
from setuptools.command.build_ext import build_ext
from setuptools.command.test import test as TestCommand
from shutil import copyfile, copymode

PROJECT_NAME = "pinetree"
VERSION = "0.3.0"

class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=''):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)

class CMakeBuild(build_ext):
    def run(self):
        try:
            out = subprocess.check_output(['cmake', '--version'])
        except:
            print("\nPinetree requires CMake to build and install.\n")
            print(
                "To install CMake, run: \n\npython3 -m pip install cmake\n\nInstallation failed. ")
            sys.exit(1)

        if platform.system() == "Windows":
            cmake_version = LooseVersion(re.search(r'version\s*([\d.]+)',
                                                   out.decode()).group(1))
            if cmake_version < '3.1.0':
                raise RuntimeError("CMake >= 3.1.0 is required on Windows")

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        extdir = os.path.abspath(
            os.path.dirname(self.get_ext_fullpath(ext.name)))
        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + extdir,
                      '-DPYTHON_EXECUTABLE=' + sys.executable]

        cfg = 'Debug' if self.debug else 'Release'
        build_args = ['--config', cfg]

        if platform.system() == "Windows":
            cmake_args += ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}'.format(
                cfg.upper(),
                extdir)]
            if sys.maxsize > 2**32:
                cmake_args += ['-A', 'x64']
            build_args += ['--', '/m']
        else:
            cmake_args += ['-DCMAKE_BUILD_TYPE=' + cfg]
            build_args += ['--', '-j2']

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(
            env.get('CXXFLAGS', ''),
            self.distribution.get_version())
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args,
                              cwd=self.build_temp, env=env)
        subprocess.check_call(['cmake', '--build', '.'] + build_args,
                              cwd=self.build_temp)
        test_bin = os.path.join(self.build_temp, PROJECT_NAME + '_test')
        self.copy_test_file(test_bin)
        print()  # Add an empty line for cleaner output

    def copy_test_file(self, src_file):
        '''
        Copy ``src_file`` to `tests/bin` directory, ensuring parent directory 
        exists. Messages like `creating directory /path/to/package` and
        `copying directory /src/path/to/package -> path/to/package` are 
        displayed on standard output. Adapted from scikit-build.
        '''
        # Create directory if needed
        dest_dir = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), 'tests', 'bin')
        if dest_dir != "" and not os.path.exists(dest_dir):
            print("creating directory {}".format(dest_dir))
            os.makedirs(dest_dir)

        # Copy file
        dest_file = os.path.join(dest_dir, os.path.basename(src_file))
        print("copying {} -> {}".format(src_file, dest_file))
        copyfile(src_file, dest_file)
        copymode(src_file, dest_file)



with open('README.md', encoding='utf-8') as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()


setup(
    name=PROJECT_NAME,
    version=VERSION,
    description='stochastic simulation of gene expression with site-specific translation rates',
    long_description=readme,
    long_description_content_type='text/markdown',
    author='Benjamin Jack',
    author_email='benjamin.r.jack@gmail.com',
    url='https://github.com/clauswilke/' + PROJECT_NAME,
    download_url='https://github.com/clauswilke/' + PROJECT_NAME + '/archive/v' + VERSION + '.tar.gz',
    license='MIT',
    keywords=['gene', 'codon', 'transcription',
              'translation', 'biology', 'stochastic'],
    packages=find_packages('src'),
    package_dir={'': 'src'},
    ext_modules=[CMakeExtension(PROJECT_NAME + '/core')],
    cmdclass=dict(build_ext=CMakeBuild),
    zip_safe=False,
    test_suite='tests'
)



# Test simulation
import unittest
import subprocess
import tempfile
import importlib


class MainTest(unittest.TestCase):

    def setUp(self):
        self.tempdir = tempfile.TemporaryDirectory()

    def tearDown(self):
        self.tempdir.cleanup()

    def run_test(self, prefix):
        test_mod = importlib.import_module('.models.' + prefix, 'tests')
        out_prefix = self.tempdir.name + "/" + prefix
        test_mod.execute(out_prefix)
        with open('tests/output/' + prefix + '_counts.tsv') as f:
            text = f.read()
        with open(out_prefix + '_counts.tsv') as results_file:
            results = results_file.read()
        self.assertEqual(results, text)

    def test_single_gene(self):
        self.run_test('single_gene')

    # def test_three_genes(self):
    #     self.run_test('three_genes')

    # def test_dual_polymerases(self):
    #     self.run_test('dual_polymerases')

    # def test_dual_promoters(self):
    #     self.run_test('dual_promoter')

    # def test_readthrough(self):
    #     self.run_test('readthrough')

    # def test_genome_entry(self):
    #     self.run_test('genome_entry')

    # def test_consecutive_promoters(self):
    #     self.run_test('consecutive_promoters')

    # def test_lotka_voltera(self):
    #     self.run_test('lotka_voltera')

    # def test_promoter_gene_overlap(self):
    #     self.run_test('promoter_gene_overlap')

    # def test_three_genes_runoff(self):
    #     self.run_test('three_genes_runoff')

    # def test_overlapping_genes(self):
    #     self.run_test('overlapping_genes')


if __name__ == '__main__':
    unittest.main()
